==> thunderbird_78.5.0.orig.tar.xz: thunderbird-78.5.0/comm/ldap/c-sdk/libraries/libldap/unescape.c <==
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*
 *  LIBLDAP unescape.c -- LDAP URL un-escape routines
 *  We also tolerate URLs that look like: <ldapurl> and <URL:ldapurl>
 */

#include "ldap-int.h"

static int unhex(char c);

void nsldapi_hex_unescape(char* s) {
  /*
   * Remove URL hex escapes from s... done in place.  The basic concept for
   * this routine is borrowed from the WWW library HTUnEscape() routine.
   */
  char* p;

  for (p = s; *s != '\0'; ++s) {
    if (*s == '%') {
      if (*++s == '\0') {
        break;
      }
      *p = unhex(*s) << 4;
      if (*++s == '\0') {
        break;
      }
      *p++ += unhex(*s);

    } else {
      *p++ = *s;
    }
  }

  *p = '\0';
}

static int unhex(char c) {
  return (c >= '0' && c <= '9'
              ? c - '0'
              : c >= 'A' && c <= 'F' ? c - 'A' + 10 : c - 'a' + 10);
}

==> thunderbird_91.10.0.orig.tar.xz: thunderbird-91.10.0/comm/ldap/c-sdk/libraries/libldap/unescape.c <==
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*
 *  LIBLDAP unescape.c -- LDAP URL un-escape routines
 *  We also tolerate URLs that look like: <ldapurl> and <URL:ldapurl>
 */

#include "ldap-int.h"

static int unhex(char c);

void nsldapi_hex_unescape(char* s) {
  /*
   * Remove URL hex escapes from s... done in place.  The basic concept for
   * this routine is borrowed from the WWW library HTUnEscape() routine.
   */
  char* p;

  for (p = s; *s != '\0'; ++s) {
    if (*s == '%') {
      if (*++s == '\0') {
        break;
      }
      *p = unhex(*s) << 4;
      if (*++s == '\0') {
        break;
      }
      *p++ += unhex(*s);

    } else {
      *p++ = *s;
    }
  }

  *p = '\0';
}

static int unhex(char c) {
  return (c >= '0' && c <= '9'   ? c - '0'
          : c >= 'A' && c <= 'F' ? c - 'A' + 10
                                 : c - 'a' + 10);
}
